// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "Courbe1D.h"

// les différentes courbes existantes
#include "CourbePolynomiale.h"
#include "F1_rond_F2.h"
#include "F1_plus_F2.h"
#include "CourbePolyLineaire1D.h"
#include "Courbe_expoaff.h"
#include "Courbe_un_moins_cos.h"
#include "CourbePolyLineaire1D_simpli.h"
#include "F_cyclique.h"
#include "F_cycle_add.h"
#include "F_union_1D.h"
#include "TripodeCos3phi.h"
#include "SixpodeCos3phi.h"
#include "Poly_Lagrange.h"
#include "Courbe_expo_n.h"
#include "Courbe_expo2_n.h"
#include "Courbe_relax_expo.h"
#include "Courbe_cos.h"
#include "Courbe_sin.h"
#include "TangenteHyperbolique.h"
#include "CourbePolyHermite1D.h"
#include "Courbe_ln_cosh.h"
#include "Courbe_expression_litterale_1D.h"
#include "Courbe_expression_litterale_avec_derivees_1D.h"
    // METHODES PUBLIQUES :
    
    // ---------- non virtuelle ---------
    
// ramène un pointeur sur la courbe correspondant au type de courbe passé en paramètre
Courbe1D* Courbe1D::New_Courbe1D(string& nom,EnumCourbe1D typeCourbe)
  {	// définition du pointeur de retour
    Courbe1D* courbe;
    // choix en fonction du typeCourbe
    switch (typeCourbe)
	{
		case COURBEPOLYLINEAIRE_1_D : courbe= new CourbePolyLineaire1D(nom); break;
		case COURBE_EXPOAFF : courbe= new Courbe_expoaff(nom); break;
		case COURBE_UN_MOINS_COS : courbe= new Courbe_un_moins_cos(nom); break;
		case CPL1D : courbe= new Cpl1D(nom); break;
		case COURBEPOLYNOMIALE : courbe= new CourbePolynomiale(nom); break;
		case COURBEPOLYHERMITE_1_D : courbe= new CourbePolyHermite1D(nom); break;
		case F1_ROND_F2 : courbe= new F1_rond_F2(nom); break;
		case F1_PLUS_F2 : courbe= new F1_plus_F2(nom); break;
		case F_CYCLIQUE : courbe= new F_cyclique(nom); break;
		case F_CYCLE_ADD : courbe= new F_cycle_add(nom); break;
		case F_UNION_1D : courbe= new F_union_1D(nom); break;
		case COURBE_TRIPODECOS3PHI : courbe= new TripodeCos3phi(nom); break;
		case COURBE_SIXPODECOS3PHI : courbe= new SixpodeCos3phi(nom); break;
		case COURBE_POLY_LAGRANGE : courbe= new Poly_Lagrange(nom); break;
		case COURBE_EXPO_N : courbe= new Courbe_expo_n(nom); break;
		case COURBE_EXPO2_N : courbe= new Courbe_expo2_n(nom); break;
		case COURBE_RELAX_EXPO : courbe= new Courbe_relax_expo(nom); break;
		case COURBE_COS : courbe= new Courbe_cos(nom); break;
		case COURBE_SIN : courbe= new Courbe_sin(nom); break;
		case COURBE_TANH : courbe= new TangenteHyperbolique(nom); break;
		case COURBE_LN_COSH : courbe= new Courbe_ln_cosh(nom); break;
		case COURBE_EXPRESSION_LITTERALE_1D : courbe= new Courbe_expression_litterale_1D(nom); break;
		case COURBE_EXPRESSION_LITTERALE_AVEC_DERIVEE_1D : courbe= new Courbe_expression_litterale_avec_derivees_1D(nom); break;
		default :
	     {  cout << "\nErreur : valeur incorrecte du type de Courbe1D : "
			     << typeCourbe << "\n";
			cout << "\n New_Courbe1D(EnumCourbe1D typeCourbe) \n";
			Sortie(1);
		  }
	};
	return courbe;
   };
    
// ramène un pointeur sur une courbe copie de celle passée en paramètre
// IMPORTANT : il y a création d'une courbe (utilisation d'un new)
Courbe1D* Courbe1D::New_Courbe1D(const Courbe1D& Co)
  {	// définition du pointeur de retour
    Courbe1D* courbe;
    // choix en fonction du typeCourbe
    switch (Co.Type_courbe())
	{
		case COURBEPOLYLINEAIRE_1_D : courbe= new CourbePolyLineaire1D( Co); break;
		case COURBE_EXPOAFF : courbe= new Courbe_expoaff(Co); break;
		case COURBE_UN_MOINS_COS : courbe= new Courbe_un_moins_cos(Co); break;
		case CPL1D : courbe= new Cpl1D(Co); break;
		case COURBEPOLYNOMIALE : courbe= new CourbePolynomiale(Co); break;
		case COURBEPOLYHERMITE_1_D : courbe= new CourbePolyHermite1D(Co); break;
		case F1_ROND_F2 : courbe= new F1_rond_F2(Co); break;
		case F1_PLUS_F2 : courbe= new F1_plus_F2(Co); break;
		case F_CYCLIQUE : courbe= new F_cyclique(Co); break;
		case F_CYCLE_ADD : courbe= new F_cycle_add(Co); break;
		case F_UNION_1D : courbe= new F_union_1D(Co); break;
		case COURBE_TRIPODECOS3PHI : courbe= new TripodeCos3phi(Co); break;
		case COURBE_SIXPODECOS3PHI : courbe= new SixpodeCos3phi(Co); break;
		case COURBE_POLY_LAGRANGE : courbe= new Poly_Lagrange(Co); break;
		case COURBE_EXPO_N : courbe= new Courbe_expo_n(Co); break;
		case COURBE_EXPO2_N : courbe= new Courbe_expo2_n(Co); break;
		case COURBE_RELAX_EXPO : courbe= new Courbe_relax_expo(Co); break;
		case COURBE_COS : courbe= new Courbe_cos(Co); break;
		case COURBE_SIN : courbe= new Courbe_sin(Co); break;
		case COURBE_TANH : courbe= new TangenteHyperbolique(Co); break;
		case COURBE_LN_COSH : courbe= new Courbe_ln_cosh(Co); break;
		case COURBE_EXPRESSION_LITTERALE_1D : courbe= new Courbe_expression_litterale_1D(Co); break;
		case COURBE_EXPRESSION_LITTERALE_AVEC_DERIVEE_1D : courbe= new Courbe_expression_litterale_avec_derivees_1D(Co); break;
		default :
	     {  cout << "\nErreur : valeur incorrecte du type de Courbe1D : "
			     << Co.Type_courbe() << "\n";
			cout << "\n New_Courbe1D(const Courbe1D& Co) \n";
			Sortie(1);
		  }
	};
	return courbe;
   };
    
// ramène la liste des identificateurs de courbes actuellement disponibles
list <EnumCourbe1D> Courbe1D::Liste_courbe_disponible()
  {	// définition de la liste de retour
    list <EnumCourbe1D> list_ret;
    // remplissage de la liste 
    list_ret.push_back(COURBEPOLYLINEAIRE_1_D);
    list_ret.push_back(COURBE_EXPOAFF);
    list_ret.push_back(COURBE_UN_MOINS_COS);
    list_ret.push_back(CPL1D);
    list_ret.push_back(COURBEPOLYNOMIALE);
    list_ret.push_back(COURBEPOLYHERMITE_1_D);
    list_ret.push_back(F1_ROND_F2);
    list_ret.push_back(F1_PLUS_F2);
    list_ret.push_back(F_CYCLIQUE);
    list_ret.push_back(F_CYCLE_ADD);
    list_ret.push_back(F_UNION_1D);
    list_ret.push_back(COURBE_TRIPODECOS3PHI);
    list_ret.push_back(COURBE_SIXPODECOS3PHI);
    list_ret.push_back(COURBE_POLY_LAGRANGE);
    list_ret.push_back(COURBE_EXPO_N);
    list_ret.push_back(COURBE_EXPO2_N);
    list_ret.push_back(COURBE_RELAX_EXPO);
    list_ret.push_back(COURBE_COS);
    list_ret.push_back(COURBE_SIN);
    list_ret.push_back(COURBE_TANH);
    list_ret.push_back(COURBE_LN_COSH);
    list_ret.push_back(COURBE_EXPRESSION_LITTERALE_1D);
    list_ret.push_back(COURBE_EXPRESSION_LITTERALE_AVEC_DERIVEE_1D);
    // retour de la liste
	return list_ret;
   };
    
// 2) retourne une liste de nom correspondant aux noms de courbes dont dépend *this
list <string>& Courbe1D::ListDependanceCourbes(list <string>& lico) const 
   { // par défaut, ne dépend pas de courbes, donc on ramène une liste vide
     // ce sera surchargé dans les classes qui elles éventuellement dépendent de quelque chose
     if (lico.size() != 0)
        lico.clear();
     return lico; 
   };
   
    // ---------- METHODES PROTEGEES :-------------------
    
// ramène true si les variables de la classe mère sont complèté
bool Courbe1D::Complet_var() const
  { bool ret = true;
    if (typeCourbe == AUCUNE_COURBE1D) ret = false;
    if (nom_ref == "") ret = false;
    return ret;
   };
