// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : Courbe_un_moins_cos.h
// classe  : Courbe_un_moins_cos


/************************************************************************
 *     DATE:        19/01/2001                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant le calcul d'une fonction 1D   1D       *
 *             de type : c*(1-cos((x-a)Pi/(b-a)) entre a et b,          *
 *             en dessous de a, -> 0 et au dessus de b -> c             *
 *             ainsi qu'un certain nombre d'information                 *
 *             supplémentaires telles que dérivées.                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef COURBE_UN_MOINS_COS_1_D_H
#define COURBE_UN_MOINS_COS_1_D_H

#include "Courbe1D.h"
/// @addtogroup Les_courbes_1D
///  @{
///

/**
*
*     BUT:    Classe permettant le calcul d'une fonction 1D   1D
*             de type : c*(1-cos((x-a)Pi/(b-a)) entre a et b,
*             en dessous de a, -> 0 et au dessus de b -> c
*             ainsi qu'un certain nombre d'information
*             supplémentaires telles que dérivées.
*
*
* \author    Gérard Rio
* \version   1.0
* \date      19/01/2001
* \brief       Classe permettant le calcul d'une fonction 1D de type : f(x) = c*(1-cos((x-a)Pi/(b-a))
*
*/

class Courbe_un_moins_cos : public Courbe1D
{
  public :

    // CONSTRUCTEURS :
    Courbe_un_moins_cos(string nom = "");
    
    // de copie
    Courbe_un_moins_cos(const Courbe_un_moins_cos& Co);
    Courbe_un_moins_cos(const Courbe1D& Co);
    
    // DESTRUCTEUR :
    ~Courbe_un_moins_cos();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
	void Affiche() const ;
    // ramène true si ok, false sinon
    bool Complet_courbe()const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_courbes(const string& nom, UtilLecture * );
    
    // def info fichier de commande
    void Info_commande_Courbes1D(UtilLecture & entreePrinc) ;
    
    // ramène la valeur 
    
    double Valeur(double x)  ;

    // ramène la valeur et la dérivée en paramètre
    Courbe1D::ValDer Valeur_Et_derivee(double x)  ;
    
    // ramène la dérivée 
    double Derivee(double x)  ;
    
    // ramène la valeur et les dérivées première et seconde en paramètre
    Courbe1D::ValDer2 Valeur_Et_der12(double x);
        
    // ramène la dérivée seconde
    double Der_sec(double x);
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
    Courbe1D::Valbool  Valeur_stricte(double x)  ;
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
    Courbe1D::ValDerbool  Valeur_Et_derivee_stricte(double x)  ;
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Courbes1D(ofstream& sort,const Enum_IO_XML enu) ;
     

    
    
  protected :  
    // VARIABLES PROTEGEES :
    double ax,bx,cx; // les coefficients de la loi
    
    // METHODES PROTEGEES :

 };
 /// @}  // end of group
#endif  
