// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : F1_rond_F2.h
// classe  : F1_rond_F2


/************************************************************************
 *     DATE:        14/10/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant le calcul d'une fonction composée      *
 *             F(x)= F1(F2(x))=F1 o F2 (x)                              *
 *             ainsi qu'un certain nombre d'information                 *
 *             supplémentaires telles que dérivées.                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef F1_ROND_F2_1_D_H
#define F1_ROND_F2_1_D_H

#include "Courbe1D.h"
#include "Tableau_T.h"
/// @addtogroup Les_courbes_1D
///  @{
///

/**
*
*     BUT:    Classe permettant le calcul d'une fonction composé 1D:
*             F(x)= F1(F2(x))=F1 o F2 (x)
*             ainsi qu'un certain nombre d'information
*             supplémentaires telles que dérivées.
*
*
* \author    Gérard Rio
* \version   1.0
* \date      19/01/2001
* \brief       Classe permettant le calcul d'une fonction 1D composé : F(x)= F1(F2(x))=F1 o F2 (x)
*
*/

class F1_rond_F2 : public Courbe1D
{
  public :

    // CONSTRUCTEURS :
    F1_rond_F2(string nom = "");
    // constructeur fonction de deux courbes existantes et d'un nom
    F1_rond_F2(Courbe1D* FF1, Courbe1D* FF2, string nom = "");
    
    // de copie
    F1_rond_F2(const F1_rond_F2& Co);
    F1_rond_F2(const Courbe1D& Co);
    
    // DESTRUCTEUR :
    ~F1_rond_F2();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
	void Affiche() const ;
    // ramène true si ok, false sinon
    bool Complet_courbe()const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_courbes(const string& nom, UtilLecture * );
    // dans le cas où les courbes membres sont des courbes externes
    // fonction pour les définir
    // les courbes sont défini en interne que si les courbes arguments sont elles même
    // des courbes locales. c'est-à-dire si FF1->NomCourbe() ="_" alors on recrée une courbe
    // interne avec new pour F1, sinon F1=FF1 et pas de création; idem pour f2
    // dans le cas où FF1 ou FF1 est NULL on passe, pas de traitement pour ce pointeur
    void DefCourbesMembres(Courbe1D* FF1, Courbe1D* FF2);
    
    // établir le lien entre la courbe et des courbes déjà existantes dont
    // on connait que le nom
    // permet ainsi de complèter la courbe
    // 1) renseigne si la courbe dépend d'autre courbe ou non
    bool DependAutreCourbes() const;
    // 2) retourne une liste de nom correspondant aux noms de courbes dont dépend *this
    list <string>& ListDependanceCourbes(list <string>& lico) const; 
    // 3) établit la connection entre la demande de *this et les courbes passées en paramètres
    void Lien_entre_courbe (list <Courbe1D *>&  liptco);
    

    // def info fichier de commande
    void Info_commande_Courbes1D(UtilLecture & entreePrinc) ;
    
    // ramène la valeur 
    
    double Valeur(double x)  ;

    // ramène la valeur et la dérivée en paramètre
    Courbe1D::ValDer Valeur_Et_derivee(double x)  ;
    
    // ramène la dérivée 
    double Derivee(double x)  ;
    
    // ramène la valeur et les dérivées première et seconde en paramètre
    Courbe1D::ValDer2 Valeur_Et_der12(double x);
        
    // ramène la dérivée seconde
    double Der_sec(double x);
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
    Courbe1D::Valbool  Valeur_stricte(double x)  ;
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
    Courbe1D::ValDerbool  Valeur_Et_derivee_stricte(double x)  ;
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Courbes1D(ofstream& sort,const Enum_IO_XML enu) ;
     

    
    
  protected :  
    // VARIABLES PROTEGEES :
    Courbe1D* F1; // première fonction
    Courbe1D* F2; // seconde fonction
    // variable intermédiaires pour la lecture en deux temps
    // servent également ensuite pour dire si F1 ou F2 sont interne ou pas
    // s'ils sont interne -> nom_courbei="i_interne_i", sinon ="e_externe_e"
    string nom_courbe1,nom_courbe2;

    // METHODES PROTEGEES :

 };
 /// @}  // end of group
#endif  
