// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : F_nD_courbe1D.h
// classe  : F_nD_courbe1D


/************************************************************************
 *     DATE:        01/06/2016                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Classe permettant d'utiliser une fonction courbe           *
 *           à l'intérieur d'une fonction nD                            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef F_ND_COURBE_1D_H
#define F_ND_COURBE_1D_H

#include "Courbe1D.h"
#include "Fonction_nD.h"
#include "Tableau_T.h"

/// @addtogroup Les_fonctions_nD
///  @{
///

/**
*
*
* \author    Gérard Rio
* \version   1.0
* \date      01/06/2016
* \brief       Classe permettant d'utiliser une fonction courbe  à l'intérieur d'une fonction nD .
*
*/

class F_nD_courbe1D : public Fonction_nD
{
  public :
 
    // CONSTRUCTEURS :
    // constructeur par défaut
    F_nD_courbe1D(string nom = "");
    
    // de copie
    F_nD_courbe1D(const F_nD_courbe1D& Co);
    // de copie à partir d'une instance générale
    F_nD_courbe1D(const Fonction_nD& Coo);
 
    // DESTRUCTEUR :
    ~F_nD_courbe1D();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
 
    // Surcharge de l'operateur = : realise l'egalite de deux fonctions
    Fonction_nD& operator= (const Fonction_nD& elt);
 
    // affichage de la courbe
    // = 0, 1 ou 2 (le plus précis)
    void Affiche(int niveau = 0) const ;
    // ramène true si ok, false sinon
    bool Complet_Fonction(bool affichage = true)const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la Fonction
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_Fonction_nD(const string& nom, UtilLecture * );
 
    // mise à jour des variables globales: en fonction de l'apparition de nouvelles variables
    // globales en cours de calcul
    virtual void Mise_a_jour_variables_globales();
 
    // def info fichier de commande
    void Info_commande_Fonctions_nD(UtilLecture & entreePrinc) ;
 
//    // calcul  des valeurs de la fonction, retour d'un tableau de scalaires
//    // les différents tableaux doivent contenir toutes les informations nécessaires pour l'appel
//    // NB: il peut y avoir plus d'info que nécessaire
//    // l'ordre des paramètres doit respecter celui donné par les fonctions Index_dans_tableau()
//    virtual Tableau <double> & Val_FnD_Evoluee(Tableau <double >* xi,Tableau <Coordonnee> * tab_coor
//                                          ,Tableau <TenseurBB* >* tab_tensBB );

//    // calcul équivalent, mais pour des paramètres de type ddl enum étendu et/ou type quelconque
//    // pour que l'appel à cette méthode soit correcte, il faut que la dimension de t_enu + celle de tqi
//    // soit identique à celle du tableau Nom_variables() : en fait t_enu et tqi doivent représenter les variables
//    virtual Tableau <double> & Valeur_FnD(Tableau <Ddl_etendu> * t_enu,Tableau <TypeQuelconque >* tqi);
 
    // ramène le nombre de composantes de la fonction
    int NbComposante() const {return 1;};
 
    // établir le lien entre la fonction et une courbe ou une fonction déjà existante dont
    // on connait que le nom
    // permet ainsi de complèter la fonction
    // 1) renseigne si la courbe dépend d'une autre courbe ou non
    bool DependAutreFoncCourbes() const;
    // 2) retourne une liste de nom correspondant aux noms de courbes dont dépend *this
    list <string>& ListDependanceCourbes(list <string>& lico) const;
    // 3) retourne une liste de nom correspondant aux noms de fonctions dont dépend *this
    // ici  cette fonction n'a pas cours, utilisation de la fonction par défaut
    //    list <string>& ListDependanceFonctions(list <string>& lifo) const;
    // 4) établit la connection entre la demande de *this et les courbes et fonctions passées en paramètres
    void Lien_entre_fonc_courbe (list <Fonction_nD *>&  liptfonc,list <Courbe1D *>&  liptco);
 
 
	//----- lecture écriture de restart -----
	   // cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	   void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Fonctions_nD(ofstream& sort,const Enum_IO_XML enu) ;
     

    
    
  protected :  
    // VARIABLES PROTEGEES :
    Courbe1D*  c_Fi; //  courbe 1D
    // variable intermédiaires pour la lecture en deux temps
    // servent également ensuite pour dire si F1 est interne ou pas
    // si elle est  interne -> nom_courbei="i_interne_i", sinon ="e_externe_e"
    string nom_courbe1;
    // donc les différentes étapes possibles sont les suivantes:
    // nom_courbe1 = ""; : après le constructeur par défaut
    // nom_courbe1 = "chaine" , avec c_Fi = NULL : la courbe interne n'est pas encore
    //                                              définie, et son nom = chaine
    // nom_courbe1 = "i_interne_i", avec c_Fi non NULL : on a une courbe interne définie en interne
    // nom_courbe1 = "e_externe_e", avec c_Fi non NULL : on a une courbe interne
    //                                                   qui pointe sur une courbe définie par ailleurs

    // égal à celui du tableau nom_variables
    Tableau <double> tab_ret; // le tableau de retour

 
    // METHODES PROTEGEES :
    // dans le cas où la courbe membre est une courbe externe
    // méthode pour la définir
    // la courbea est défini en interne que si la courbe argument est elle même
    // une courbe locale. c'est-à-dire si c_Fi->NomCourbe() ="_" alors on recrée une courbe
    // interne avec new pour c_Fi, sinon c_Fi=cc_Fi et pas de création;
    // dans le cas où cc_Fi est NULL on passe, pas de traitement pour ce pointeur
    void DefFoncCourbeMembre(Courbe1D*  cc_Fi);
 
    // calcul  des valeurs de la fonction, retour d'un tableau de scalaires
    virtual Tableau <double> & Valeur_FnD_interne(Tableau <double >* val_ddl_enum);

    // calcul des valeurs de la fonction, dans le cas où les variables
    // sont des grandeurs globales
    virtual Tableau <double> & Valeur_pour_variables_globales_interne();

 };
 /// @}  // end of group

#endif  
