// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : F_union_1D.h
// classe  : F_union_1D


/************************************************************************
 *     DATE:        14/10/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Classe permettant le calcul d'une fonction égale à         *
 *           l'union d'une suite de fonctions définies sur des segments *
 *           contigus de manière à couvrir un segment global.           *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef F1_UNION_1_D_H
#define F1_UNION_1_D_H

#include "Courbe1D.h"
#include "Tableau_T.h"
/// @addtogroup Les_courbes_1D
///  @{
///

/**
*
*     BUT:  Classe permettant le calcul d'une fonction égale à
*           l'union d'une suite de fonctions définies sur des segments
*           contigus de manière à couvrir un segment global.
*
*
* \author    Gérard Rio
* \version   1.0
* \date      14/10/2004
* \brief       Classe permettant le calcul d'une fonction égale à l'union d'une suite de fonctions définies sur des segments contigus de manière à couvrir un segment global.
*
*/

class F_union_1D : public Courbe1D
{
  public :

    // CONSTRUCTEURS :
    // constructeur par défaut
    F_union_1D(string nom = "");
    
    // de copie
    F_union_1D(const F_union_1D& Co);
    F_union_1D(const Courbe1D& Co);
    
    // DESTRUCTEUR :
    ~F_union_1D();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
	   void Affiche() const ;
    // ramène true si ok, false sinon
    bool Complet_courbe()const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_courbes(const string& nom, UtilLecture * );
    
    // établir le lien entre la courbe et une courbe déjà existante dont
    // on connait que le nom
    // permet ainsi de complèter la courbe
    // 1) renseigne si la courbe dépend d'une autre courbe ou non
    bool DependAutreCourbes() const;
    // 2) retourne une liste de nom correspondant aux noms de courbes dont dépend *this
    list <string>& ListDependanceCourbes(list <string>& lico) const; 
    // 3) établit la connection entre la demande de *this et les courbes passées en paramètres
    void Lien_entre_courbe (list <Courbe1D *>&  liptco);
    

    // def info fichier de commande
    void Info_commande_Courbes1D(UtilLecture & entreePrinc) ;
    
    // ramène la valeur 
    
    double Valeur(double x)  ;

    // ramène la valeur et la dérivée en paramètre
    Courbe1D::ValDer Valeur_Et_derivee(double x)  ;
    
    // ramène la dérivée 
    double Derivee(double x)  ;
    
    // ramène la valeur et les dérivées première et seconde en paramètre
    Courbe1D::ValDer2 Valeur_Et_der12(double x);
        
    // ramène la dérivée seconde
    double Der_sec(double x);
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
    Courbe1D::Valbool  Valeur_stricte(double x)  ;
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
    Courbe1D::ValDerbool  Valeur_Et_derivee_stricte(double x)  ;
    
	   //----- lecture écriture de restart -----
	   // cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	   void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Courbes1D(ofstream& sort,const Enum_IO_XML enu) ;
     

    
    
  protected :  
    // VARIABLES PROTEGEES :
    Tableau <Courbe1D* > Fi; //  fonctions de base
    Tableau <double > Xi; // définit les intervallees de définition des Fi
        // Fi(i) est défini sur [Xi(i),Xi(i+1)]
    // variable intermédiaires pour la lecture en deux temps
    // servent également ensuite pour dire si Fi est interne ou pas
    // si elle est  interne -> nom_courbei="i_interne_i", sinon ="e_externe_e"
    Tableau <string > nom_courbei;
    int indice_precedant; // position précédente
    
    // METHODES PROTEGEES :
    // dans le cas où les courbes membres sont des  courbes externes
    // fonction pour les définir
    // les courbes sont défini en interne que si les courbes argument sont elles même
    // des courbes locales. c'est-à-dire si FFi(i)->NomCourbe() ="_" alors on recrée une courbe
    // interne avec new pour Fi(i), sinon Fi(i)=FFi(i) et pas de création; 
    // dans le cas où FFi(i) est NULL on passe, pas de traitement pour ce pointeur
    void DefCourbesMembres(Tableau <Courbe1D* >& FFi);

 };
 /// @}  // end of group
#endif  
