// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : Fonction_externe_nD.h
// classe  : Fonction_externe_nD


/************************************************************************
 *     DATE:        01/06/2016                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant le calcul d'une fonction nD            *
 *             externe, définie par l'utilisateur, via 2 pipes nommés.  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef FONCTION_EXTERNE_ND_H
#define FONCTION_EXTERNE_ND_H

#include "Fonction_nD.h"

/// @addtogroup Les_fonctions_nD
///  @{
///

/// définition d'une union qui lie les réels, les entiers et les caractères
union Tab_car_double_int_1
  {  char    tampon[928];
     double  x[116];
     int     n[232];
  } ;
/// @}  // end of group

/// @addtogroup Les_fonctions_nD
///  @{
///

/**
*
*
* \author    Gérard Rio
* \version   1.0
* \date      01/06/2016
* \brief       Classe permettant le calcul d'une fonction nD externe, définie par l'utilisateur, via 2 pipes nommés.
*
*/

class Fonction_externe_nD : public Fonction_nD
{
  public :

    // CONSTRUCTEURS :
    // def uniquement du nom, et init par défaut -> après ce constructeur
    // la fonction doit-être renseignée pour être utilisable
    Fonction_externe_nD(string nom = "");
    
    // def de toutes les grandeurs -> donne une fonction utilisable
    Fonction_externe_nD
         ( string nom_ref  // nom de ref de la fonction
          ,Tableau <string >& nom_variables // les variables non globales
          ,Tableau <Enum_GrandeurGlobale >& enu_variables_globale // enu globaux
          ,Tableau <string >& nom_variables_globales // idem sous forme de strings
          ,int nb_double_ret);  // nombre de grandeur en retour de fonction externe
    
    // de copie
    Fonction_externe_nD(const Fonction_externe_nD& Co);
    Fonction_externe_nD(const Fonction_nD& Co);
    
    // DESTRUCTEUR :
    ~Fonction_externe_nD();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
 
    // Surcharge de l'operateur = : realise l'egalite de deux fonctions
    Fonction_nD& operator= (const Fonction_nD& elt);
 
    // affichage de la Fonction
    // = 0, 1 ou 2 (le plus précis)
    void Affiche(int niveau = 0) const ;
    // ramène true si ok, false sinon
    bool Complet_Fonction(bool affichage = true)const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la Fonction
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_Fonction_nD(const string& nom, UtilLecture * );
 
    // mise à jour des variables globales: en fonction de l'apparition de nouvelles variables
    // globales en cours de calcul
    virtual void Mise_a_jour_variables_globales();
 
    // def info fichier de commande
    void Info_commande_Fonctions_nD(UtilLecture & entreePrinc) ;    
 
//   --- erreur il faut utiliser la fonction mère
//    // ramène le nombre de variable de la fonction
//    int NbVariable()const {return tab_fVal.Taille();};
 
    // ramène le nombre de composantes de la fonction
    int NbComposante() const {return tab_ret.Taille();};
 
	//----- lecture écriture de restart -----
	   // cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	   void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu
    virtual void SchemaXML_Fonctions_nD(ofstream& sort,const Enum_IO_XML enu);
    
  protected :  
    // VARIABLES PROTEGEES :
 
    
    // -- variables pour les tubes nommés ----------
    
    // nom du tube nommé pour l'envoi des données
    string envoi;
    // nom du tube nommé pour la reception des données
    string reception;
    int passage; // test pour nombre de passage dans l'appel
    Tab_car_double_int_1 t_car_x_n; // tableau de caractères, réels et entiers
    

 
//    mu::Parser p; // définition d'une instance de parser
    Tableau <double> tab_fVal; // les variables internes dont le nombre doit être
      // égal à celui du tableau nom_variables + les enum globaux + les noms globaux
    Tableau <double> tab_ret; // le tableau de retour
 
    // METHODES PROTEGEES :

    // calcul  des valeurs de la fonction, retour d'un tableau de scalaires
    virtual Tableau <double> & Valeur_FnD_interne(Tableau <double >* xi);

    // calcul des valeurs de la fonction, dans le cas où les variables
    // sont "tous" des grandeurs globales
    virtual Tableau <double> & Valeur_pour_variables_globales_interne();
    
    // initialisation des pipes
    void InitialisationPipesNommes();
    
    // lecture des données sur le pipe
    void LectureResultatsPipe();
    // écriture des données sur le pipe
    void EcritureDonneesPipe();


    // changement du nom des pipes
    void Change_nom_pipe_envoi(const string& env) {envoi=env;};
    void Change_nom_pipe_reception(const string& recep) {reception=recep;};


 };
 /// @}  // end of group

#endif  
