// FICHIER : Coordonnee1.h
// CLASSE : Coordonnee1

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
*     DATE:        23/01/97                                            *
*                                                                $     *
*     AUTEUR:      G RIO                                               *
*                                                                $     *
*     PROJET:      Herezh++                                            *
*                                                                $     *
************************************************************************
* BUT: Les classes Coordonnee1 servent a la localisation dans l'espace *
*     1D des objets tels que les noeuds ou les points. Ces classes     *
*     dérivent des Classes génériques Coordonnee.                      *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
************************************************************************/

                                                    

#ifndef COORDONNEE1_H
#define COORDONNEE1_H


//#include "Debug.h"
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "Sortie.h"
#include "Coordonnee.h"

/** @defgroup Les_classes_coordonnee1
*
* BUT: Les classes Coordonnee1 servent a la localisation dans l'espace
*     1D des objets tels que les noeuds ou les points. Ces classes
*     dérivent des Classes génériques Coordonnee.
* \author    Gérard Rio
* \version   1.0
* \date       23/01/97
* \brief       Définition des classes de type Coordonnee1, en coordonnées sans variance (ex:  absolues)
*  ou en coordonnées locales c'est-à-dire en coordonnées covariantes ou contravariantes. Ces classes sont une spécialisation 1D des classes générales Coordonnee
 *
 */



/// @addtogroup Les_classes_coordonnee1
///  @{
///

//==============================================================================
//!    cas des coordonnées simples sans variance
//==============================================================================

class Coordonnee1 : public Coordonnee
{ 

	public :
  /// Surcharge de l'operateur * : multiplication entre un scalaire et des coordonnees
  inline friend Coordonnee1 operator* (double val,const Coordonnee1& c);
       
        // CONSTRUCTEURS :
		
  /*! \brief
  // Constructeur par defaut
		// il y a initialisation des coordonnées à zéro par défaut
  */
		Coordonnee1 ();
		/*! \brief
  // Constructeur suivant un booleen
		// quelque soit la valeur du booleen il n'y a pas initialisation des coordonnées 
		// ceci  pour aller plus vite par rapport au constructeur par défaut
  */
		Coordonnee1 (bool test );
		/// Constructeur pour une localisation monodimensionnelle
		Coordonnee1 (double x);
		/*! \brief
  // constructeur fonction d'une adresse memoire ou sont stockee les coordonnees
		//  ( l'existance de la place mémoire est a la charge
		//  de l'utilisateur !!).
  */
		Coordonnee1 (double* t);
		/// Constructeur fonction d'un vecteur qui doit avoir une  1
		Coordonnee1 ( const Vecteur& vec);
		/// Constructeur de copie
		Coordonnee1 (const Coordonnee1& c);
		/// Constructeur de copie pour une instance indiférenciée
		Coordonnee1 (const Coordonnee& c);
				
		/// DESTRUCTEUR :
		virtual ~Coordonnee1 () ;
		
		// METHODES :
		
		/// Renvoie le nombre de coordonnees
		int Dimension () const ;
		
  /*! \brief
  // Desallocation de la place memoire allouee
  // fonction définie dans la classe mère générique mais qui n'a pas de
  // sens ici
  */
  void Libere ();
        
		// Renvoie le nombre de coordonnees
		//int Dimension () const ;
  
  /*! \brief
  // changement de la dimension
  // fonction définie dans la classe mère générique mais qui n'a pas de
  // sens ici, affiche un message d'erreur
  */
		void Change_dim(int dim);
		
		/// Surcharge de l'operateur = : realise l'affectation entre deux points
		Coordonnee1& operator= (const Coordonnee1& c);
		
		/// Surcharge de l'operateur - : renvoie l'oppose d'un point
		Coordonnee1 operator- () const ;
		
		/*! \brief
  // Surcharge de l'operateur - : realise la soustraction des
		// coordonnees de deux points
  */
		Coordonnee1 operator- (const Coordonnee1& c) const ;
		
		/*! \brief
  // Surcharge de l'operateur + : realise l'addition des
		// coordonnees de deux points
  */
		Coordonnee1 operator+ (const Coordonnee1& c) const ;
		
		/// Surcharge de l'operateur +=
		void operator+= (const Coordonnee1& c);

		/// Surcharge de l'operateur -=
		void operator-= (const Coordonnee1& c);

		/// Surcharge de l'operateur *=
		void operator*= (double val);
		
		/// Surcharge de l'operateur * : multiplication de coordonnees par un scalaire
		Coordonnee1 operator* (double val) const ;
		
		/// Surcharge de l'operateur * : produit scalaire entre coordonnees
		double operator* (const Coordonnee1& c) const ;
		
		/// Surcharge de l'operateur / : division de coordonnees par un scalaire
		Coordonnee1 operator/ (double val) const  ;
		
		/// Surcharge de l'operateur /= : division de coordonnees par un scalaire
		void operator/= (double val) ;

		/*! \brief
  // Surcharge de l'operateur == : test d'egalite
		// Renvoie 1 si les deux positions sont identiques
		// Renvoie 0 sinon
  */
		int operator== (const Coordonnee1& c) const;

  ///  conversion en Vecteur
  Vecteur Vect() const ;
  
  /// mise a zero des coordonnées
		void Zero() ; 
		
		/// Calcul de la norme euclidienne des composantes du point
		double Norme ()  const ;
		
		/// norme le vecteur coordonnée
		Coordonnee1& Normer ();
		
		/// somme de tous les composantes
		double Somme() const ;
  /// sortie du schemaXML: en fonction de enu
  static void SchemaXML_Coordonnee(ofstream& sort,const Enum_IO_XML enu) ;
		
	protected :
	
		double coord1[1];
		
};
/// @}  // end of group

/// @addtogroup Les_classes_coordonnee1
///  @{
///

class Coordonnee1B;  // défini par la suite ( nécessaire pour le produit scalaire)

//==============================================================================
//!    cas des coordonnées contravariantes
//==============================================================================

class Coordonnee1H : public CoordonneeH
{ 

	public :
	 friend class Coordonnee1B;
  
  /// Surcharge de l'operateur * : multiplication entre un scalaire et des coordonnees
  inline friend Coordonnee1H operator* (double val,const Coordonnee1H& c);
     
      // CONSTRUCTEURS :
		
		/*! \brief
  // Constructeur par defaut
		// il y a initialisation des coordonnées à zéro par défaut
  */
		Coordonnee1H ();
		/*! \brief
  // Constructeur suivant un booleen
		// quelque soit la valeur du booleen il n'y a pas initialisation des coordonnées 
		// ceci  pour aller plus vite par rapport au constructeur par défaut
  */
		Coordonnee1H (bool test );
		/// Constructeur pour une localisation monodimensionnelle
		Coordonnee1H (double x);
		/*! \brief
  // constructeur fonction d'une adresse memoire ou sont stockee les coordonnees
		//  ( l'existance de la place mémoire est a la charge
		//  de l'utilisateur !!).
  */
		Coordonnee1H (double* t);
		/// Constructeur fonction d'un vecteur qui doit avoir une  1
		Coordonnee1H ( const Vecteur& vec);
		/// Constructeur de copie
		Coordonnee1H (const Coordonnee1H& c);
		/// Constructeur de copie pour une instance indiférenciée
		Coordonnee1H (const CoordonneeH& c);
				
		/// DESTRUCTEUR :
		virtual ~Coordonnee1H () ;
		
		// METHODES :
		
		/// Renvoie le nombre de coordonnees
		int Dimension () const ;
		
  /*! \brief
  // Desallocation de la place memoire allouee
  // fonction définie dans la classe mère générique mais qui n'a pas de
  // sens ici
  */
  void Libere ();
        
		// Renvoie le nombre de coordonnees
		//int Dimension () const ;
  
  /*! \brief
  // changement de la dimension
  // fonction définie dans la classe mère générique mais qui n'a pas de
  // sens ici, affiche un message d'erreur
  */
		void Change_dim(int dim);
		
		/// Surcharge de l'operateur = : realise l'affectation entre deux points
		Coordonnee1H& operator= (const Coordonnee1H& c);
		
		/// Surcharge de l'operateur - : renvoie l'oppose d'un point
		Coordonnee1H operator- () const ;
		
		/*! \brief
  // Surcharge de l'operateur - : realise la soustraction des
		// coordonnees de deux points
  */
		Coordonnee1H operator- (const Coordonnee1H& c) const ;
		
		/*! \brief
  // Surcharge de l'operateur + : realise l'addition des
		// coordonnees de deux points
  */
		Coordonnee1H operator+ (const Coordonnee1H& c) const ;
		
		/// Surcharge de l'operateur +=
		void operator+= (const Coordonnee1H& c);

		/// Surcharge de l'operateur -=
		void operator-= (const Coordonnee1H& c);

		/// Surcharge de l'operateur *=
		void operator*= (double val);
		
		/// Surcharge de l'operateur * : multiplication de coordonnees par un scalaire
		Coordonnee1H operator* (double val) const ;
		
		/// Surcharge de l'operateur * : produit scalaire entre coordonnees
		double operator* (const Coordonnee1B& c) const ;
		
		///  produit scalaire entre coordonnees contravariantes et contravariantes
		double ScalHH(const Coordonnee1H& c) const ;

		/// Surcharge de l'operateur / : division de coordonnees par un scalaire
		Coordonnee1H operator/ (double val) const  ;
		
		/// Surcharge de l'operateur /= : division de coordonnees par un scalaire
		void operator/= (double val) ;

		/*! \brief
  // Surcharge de l'operateur == : test d'egalite
		// Renvoie 1 si les deux positions sont identiques
		// Renvoie 0 sinon
  */
		int operator== (const Coordonnee1H& c) const;

  ///  conversion en Vecteur
  Vecteur Vect() const ;
  
  /// mise a zero des coordonnées
		void Zero() ; 
		
		/// Calcul de la norme euclidienne des composantes du point
		double Norme ()  const ;
		
		/// norme le vecteur coordonnée
		Coordonnee1H& Normer ();
		
		/// somme de tous les composantes
		double Somme() const ;
  /// sortie du schemaXML: en fonction de enu
  static void SchemaXML_Coordonnee(ofstream& sort,const Enum_IO_XML enu) ;
		
	protected :
	
		double coord1[1];
		
};
/// @}  // end of group


/// @addtogroup Les_classes_coordonnee1
///  @{
///
//==============================================================================
//!    cas des coordonnées covariantes
//==============================================================================

class Coordonnee1B : public CoordonneeB
{ 

	public :
	 friend class Coordonnee1H;
  
  /// Surcharge de l'operateur * : multiplication entre un scalaire et des coordonnees
  inline friend Coordonnee1B operator* (double val,const Coordonnee1B& c);
     
      // CONSTRUCTEURS :
		
		/*! \brief
  // Constructeur par defaut
		// il y a initialisation des coordonnées à zéro par défaut
  */
		Coordonnee1B ();
		/*! \brief
  // Constructeur suivant un booleen
		// quelque soit la valeur du booleen il n'y a pas initialisation des coordonnées 
		// ceci  pour aller plus vite par rapport au constructeur par défaut
  */
		Coordonnee1B (bool test );
		/// Constructeur pour une localisation monodimensionnelle
		Coordonnee1B (double x);
		/*! \brief
  // constructeur fonction d'une adresse memoire ou sont stockee les coordonnees
		//  ( l'existance de la place mémoire est a la charge
		//  de l'utilisateur !!).
  */
		Coordonnee1B (double* t);
		/// Constructeur fonction d'un vecteur qui doit avoir une  1
		Coordonnee1B ( const Vecteur& vec);
		/// Constructeur de copie
		Coordonnee1B (const Coordonnee1B& c);
		/// Constructeur de copie pour une instance indiférenciée
		Coordonnee1B (const CoordonneeB& c);
				
		/// DESTRUCTEUR :
		virtual ~Coordonnee1B () ;
		
		// METHODES :
		
		/// Renvoie le nombre de coordonnees
		int Dimension () const ;
		
  /*! \brief
  // Desallocation de la place memoire allouee
  // fonction définie dans la classe mère générique mais qui n'a pas de
  // sens ici
  */
  void Libere ();
        
		// Renvoie le nombre de coordonnees
		//int Dimension () const ;
  
  /*! \brief
  // changement de la dimension
  // fonction définie dans la classe mère générique mais qui n'a pas de
  // sens ici, affiche un message d'erreur
  */
		void Change_dim(int dim);
		
		/// Surcharge de l'operateur = : realise l'affectation entre deux points
		Coordonnee1B& operator= (const Coordonnee1B& c);
		
		/// Surcharge de l'operateur - : renvoie l'oppose d'un point
		Coordonnee1B operator- () const ;
		
		/*! \brief
  // Surcharge de l'operateur - : realise la soustraction des
		// coordonnees de deux points
  */
		Coordonnee1B operator- (const Coordonnee1B& c) const ;
		
		/*! \brief
  // Surcharge de l'operateur + : realise l'addition des
		// coordonnees de deux points
  */
		Coordonnee1B operator+ (const Coordonnee1B& c) const ;
		
		/// Surcharge de l'operateur +=
		void operator+= (const Coordonnee1B& c);

		/// Surcharge de l'operateur -=
		void operator-= (const Coordonnee1B& c);

		/// Surcharge de l'operateur *=
		void operator*= (double val);
		
		/// Surcharge de l'operateur * : multiplication de coordonnees par un scalaire
		Coordonnee1B operator* (double val) const ;
		
		/// Surcharge de l'operateur * : produit scalaire entre coordonnees
		double operator* (const Coordonnee1H& c) const ;
		
		///  produit scalaire entre coordonnees covariantes et covariantes
		double ScalBB(const Coordonnee1B& c) const ;

		/// Surcharge de l'operateur / : division de coordonnees par un scalaire
		Coordonnee1B operator/ (double val) const  ;
		
		/// Surcharge de l'operateur /= : division de coordonnees par un scalaire
		void operator/= (double val) ;

		/*! \brief
  // Surcharge de l'operateur == : test d'egalite
		// Renvoie 1 si les deux positions sont identiques
		// Renvoie 0 sinon
  */
		int operator== (const Coordonnee1B& c) const;

  ///  conversion en Vecteur
  Vecteur Vect() const ;
  
  /// mise a zero des coordonnées
		void Zero() ; 
		
		/// Calcul de la norme euclidienne des composantes du point
		double Norme ()  const ;
		
		/// norme le vecteur coordonnée
		Coordonnee1B& Normer ();
		
		/// somme de tous les composantes
		double Somme() const ;
  /// sortie du schemaXML: en fonction de enu
  static void SchemaXML_Coordonnee(ofstream& sort,const Enum_IO_XML enu) ;
		
	protected :
	
		double coord1[1];
		
};
/// @}  // end of group


#ifndef MISE_AU_POINT
  #include "Coordonnee1.cc"
  #include "Coordonnee1H.cc"
  #include "Coordonnee1B.cc"
  #define  COORDONNEE1_H_deja_inclus
#endif
//#include "Vecteur.h"

#endif
