

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
*     DATE:        23/01/97                                            *
*                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
*                                                                $     *
*     PROJET:      Herezh++                                            *
*                                                                $     *
************************************************************************
*     BUT:   Definition des classes derivees de dimension 1.           *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
************************************************************************/

#ifndef Tenseur1_H
#define Tenseur1_H

//#include "Debug.h"
#include <iostream> 
#include "Tenseur.h"  
#include "PtTabRel.h"
#include "MathUtil.h"
#include "Coordonnee1.h"


/** @defgroup Les_classes_tenseurs_dim1_ordre2
*
*     BUT:   Definir les tenseurs d'ordre 2 de differentes composantes,
*     spécifiquement à la dimension 1.
*     L'objectif principal est de surcharger les differentes operations
*     classiques.
*
*   concernant le produit contracte un fois, en particulier pour les tenseurs mixtes
*   il y a contraction du 2ieme indice du premier tenseur avec le premier indice du second
*   tenseur : Aij * Bjk = C ik <-> A * B = C
*   le tenseur inverse par rapport au produit contracte est defini de la maniere suivante
*   Inverse(A) * A = Id, ainsi l'inverse d'un tenseur BH est un BH idem pour les HB
*   mais l'inverse d'un BB est un HH, et l'inverse d'un HH est un BB
*
*   NB: lorsque les tenseurs mixtes sont issues de tenseurs HH ou BB symetrique, l'ordre
*   de contraction des indices n'a pas d'importance sur le resultat !!
*
*   le produit contracte de deux tenseurs symetriques quelconques ne donne pas un tenseur
*   symetrique !!, donc par exemple la contraction d'un tenseur HB avec HH n'est pas forcement
*   symetrique. Le resultat est symetrique SEULEMENT lorsque ces operations sont effectues
*   avec le tenseur metrique.
*
* \author    Gérard Rio
* \version   1.0
* \date       23/01/97
* \brief       Définition des classes de dimension 1 de type Tenseur d'ordre 2, en coordonnées avec différentes variances.
*
*/



class TenseurHB;   // pour l'utilisation dans les produits contractes

/// @addtogroup Les_classes_tenseurs_dim1_ordre2
///  @{

//------------------------------------------------------------------
///  Definition des tenseur  derivees de dimension1.
///          cas des composantes deux fois contravariantes
//------------------------------------------------------------------
/// \author    Gérard Rio
/// \version   1.0
/// \date       23/01/97
class Tenseur1HH : public TenseurHH
{
 // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1HH &);
 // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1HH &);
  public :
    // Constructeur
    Tenseur1HH() ; // par défaut
    // initialisation de la compoante (1,1)
    Tenseur1HH(double val) ;    
    // DESTRUCTEUR :
    ~Tenseur1HH() ;
    // constructeur a partir d'une instance non differenciee  
    Tenseur1HH (const TenseurHH &); 
    // constructeur de copie  
    Tenseur1HH ( const  Tenseur1HH & B);
        
    // METHODES PUBLIQUES :
    // initialise toutes les composantes à val
    void Inita(double val);   
    // operations 
     TenseurHH & operator + ( const TenseurHH &) const ;
     void operator += ( const TenseurHH &);
     TenseurHH & operator - () const ;
     TenseurHH & operator - ( const TenseurHH &) const ;
     void  operator -= ( const TenseurHH &);
     TenseurHH & operator = ( const TenseurHH &);
     TenseurHH & operator = ( const Tenseur1HH & B) 
      { return this->operator=(*((const TenseurHH*) & B));};
     TenseurHH & operator * ( const double &) const ;
     void  operator *= ( const double &);
     TenseurHH & operator / ( const double &) const ;
     void  operator /= ( const double &);
     
     // produit contracte avec un vecteur
     CoordonneeH operator * ( const CoordonneeB & ) const ;
   
     // produit contracte contracté une fois A(i,j)*B(j,k)=A.B
     // -> donc c'est l'indice du milieu qui est contracté
     TenseurHH & operator * ( const TenseurBH &) const ;
     TenseurHB & operator * ( const TenseurBB &) const ;
     // produit contracte contracté deux fois A(i,j)*B(j,i)=A..B
     // -> on contracte d'abord l'indice du milieu puis l'indice externe
     double  operator && ( const TenseurBB &) const ;
     
     // test
     int operator == ( const TenseurHH &) const ;
     int operator != ( const TenseurHH &) const ;
    
     double  Det() const ;     // determinant de la matrice des coordonnees

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
     TenseurHH & Transpose() const ;
    
    // calcul du maximum en valeur absolu des composantes du tenseur
     double MaxiComposante() const {return Dabs(*t);};
    
    // ---- manipulation d'indice ---- -> création de nouveaux tenseurs
    virtual TenseurBB& Baisse2Indices() const;
    virtual TenseurBH& BaissePremierIndice() const;
    virtual TenseurHB& BaisseDernierIndice() const;
         
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurHH & B,bool plusZero);
     // calcul du tenseur inverse par rapport au produit contracte
     TenseurBB & Inverse() const ;

    // retourne la composante i,j en lecture/écriture
    #ifdef MISE_AU_POINT
      inline double& Coor(int i,int j)
    #else 	
         inline double& Coor(int ,int )
    #endif     	 
       { 
    #ifdef MISE_AU_POINT	 	 
       if ( (i!=1) || (j!=1) )
       { cout << "\nErreur : composante " << i <<"," << j <<"  inexistante !\n";
         cout << "Tenseur1HH::OPERATOR() (int,int ) \n";
         Sortie(1);
       };
    #endif  			
       return *t;
      }; 
    // retourne la composante i,j en lecture seule
#ifdef MISE_AU_POINT
	 inline double operator () (int i,int j) const  
#else 	
     inline double operator () (int ,int ) const
#endif     	 
	 	{ 
#ifdef MISE_AU_POINT	 	 
	 	if ( (i!=1) || (j!=1) )
			{ cout << "\nErreur : composante " << i <<"," << j <<"  inexistante !\n";
			  cout << "Tenseur1HH::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
#endif  			
			return *t;
		}; 
		
     //fonctions static définissant le produit tensoriel de deux vecteurs
     static TenseurHH & Prod_tensoriel(const CoordonneeH & aH, const CoordonneeH & bH);
	 
	 // lecture et écriture de données
     istream & Lecture(istream & entree);
     ostream & Ecriture(ostream & sort) const ;

	 
	  protected :
	    // allocator dans la liste de data
	  listdouble1Iter ipointe;  
		
 }; 
/// @}  // end of group
 
class TenseurBH;   // pour l'utilisation dans TenseurHH
      
/// @addtogroup Les_classes_tenseurs_dim1_ordre2
///  @{

//------------------------------------------------------------------
///  Definition des tenseur  derivees de dimension1.
///          cas des composantes deux fois covariantes
//------------------------------------------------------------------
/// \author    Gérard Rio
/// \version   1.0
/// \date       23/01/97
class Tenseur1BB : public TenseurBB
{
 // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1BB &);
 // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1BB &) ;
  public :
     // constructeur
    Tenseur1BB() ; // par défaut     
    // initialisation de la compoante (1,1)
    Tenseur1BB(double val) ;   
    // DESTRUCTEUR :
    ~Tenseur1BB() ;
    // constructeur a partir d'une instance non differenciee  
    Tenseur1BB ( const  TenseurBB &); 
    // constructeur de copie  
    Tenseur1BB ( const  Tenseur1BB & B);
        
    // METHODES PUBLIQUES :
    // initialise toutes les composantes à val
    void Inita(double val);   
    // operations 
    TenseurBB & operator + ( const TenseurBB &) const ;
    void operator += ( const TenseurBB &);
    TenseurBB & operator - () const ;
    TenseurBB & operator - ( const TenseurBB &) const ;
    void operator -= ( const TenseurBB &);
    TenseurBB & operator = ( const TenseurBB &) ;
    TenseurBB & operator = ( const Tenseur1BB & B) 
      { return this->operator=(*((const TenseurBB*) & B)) ;};
    TenseurBB & operator * ( const double &) const ;
    void operator *= ( const double &);
    TenseurBB & operator / ( const double &) const ;
    void operator /= ( const double &);
    
    // produit contracte avec un vecteur
    CoordonneeB operator * ( const CoordonneeH & ) const ;
     
    // produit contracte contracté une fois A(i,j)*B(j,k)=A.B
    // -> donc c'est l'indice du milieu qui est contracté
    TenseurBB & operator * ( const TenseurHB &) const ;
    TenseurBH & operator * ( const TenseurHH &) const ;
    // produit contracte contracté deux fois A(i,j)*B(j,i)=A..B
    // -> on contracte d'abord l'indice du milieu puis l'indice externe
    double  operator && ( const TenseurHH &) const ;
             
    // test
    int operator == ( const TenseurBB &) const ;
    int operator != ( const TenseurBB &) const ;
    
    double  Det() const ;     // determinant de la matrice des coordonnees
          
    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    TenseurBB & Transpose() const ; 
    
    // ---- manipulation d'indice ---- -> création de nouveaux tenseurs
    virtual TenseurHH& Monte2Indices() const ;
    virtual TenseurHB& MontePremierIndice() const ;
    virtual TenseurBH& MonteDernierIndice() const ;

    // calcul du maximum en valeur absolu des composantes du tenseur
     double MaxiComposante() const {return Dabs(*t);};
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurBB & B,bool plusZero);

    // calcul du tenseur inverse par rapport au produit contracte
    TenseurHH & Inverse() const ;                  

    // retourne la composante i,j en lecture/écriture
#ifdef MISE_AU_POINT
	 inline double& Coor(int i,int j)
#else 	
     inline double& Coor(int ,int )
#endif     	 
	 	{ 
#ifdef MISE_AU_POINT	 	 
	 	if ( (i!=1) || (j!=1) )
			{ cout << "\nErreur : composante  " << i <<"," << j <<"  inexistante !\n";
			  cout << " Tenseur1BB::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
#endif  			
			return *t;
		}; 
		
    // retourne la composante i,j en écriture seule
#ifdef MISE_AU_POINT
	 inline double operator () (int i,int j) const 
#else 	
     inline double operator () (int ,int ) const
#endif     	 
	 	{ 
#ifdef MISE_AU_POINT	 	 
	 	if ( (i!=1) || (j!=1) )
			{ cout << "\nErreur : composante  " << i <<"," << j <<"  inexistante !\n";
			  cout << " Tenseur1BB::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
#endif  			
			return *t;
		}; 

      //fonctions static définissant le produit tensoriel de deux vecteurs
      static TenseurBB & Prod_tensoriel(const CoordonneeB & aB, const CoordonneeB & bB);
	 
	 // lecture et écriture de données
     istream & Lecture(istream & entree);
     ostream & Ecriture(ostream & sort) const ;
	 
	  protected :
	    // allocator dans la liste de data
	  listdouble1Iter ipointe;  
		
};
/// @}  // end of group

class Tenseur3BH; // pour Affectation_3D_a_1D

/// @addtogroup Les_classes_tenseurs_dim1_ordre2
///  @{

//------------------------------------------------------------------
///  Definition des tenseur  derivees de dimension1.
///          cas des composantes mixtes BH
//------------------------------------------------------------------
/// \author    Gérard Rio
/// \version   1.0
/// \date       23/01/97
class Tenseur1BH : public TenseurBH
{
 // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1BH &);
 // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1BH &) ;
  friend class Tenseur3BH; // pour le passage 3D 2D: méthode Affectation_3D_a_2D(..
  public :
    // constructeur
    Tenseur1BH() ; // par défaut
    // initialisation de la compoante (1,1)
    Tenseur1BH(double val) ;     
    // DESTRUCTEUR :
    ~Tenseur1BH();
    // constructeur a partir d'une instance non differenciee  
    Tenseur1BH ( const TenseurBH &); 
    // constructeur de copie  
    Tenseur1BH ( const  Tenseur1BH & B);
        
    // METHODES PUBLIQUES :
    // initialise toutes les composantes à val
    void Inita(double val);   
    // operations 
    TenseurBH & operator + ( const TenseurBH &) const ;
    void operator += ( const TenseurBH &);
    TenseurBH & operator - () const ;
    TenseurBH & operator - ( const TenseurBH &) const ;
    void operator -= ( const TenseurBH &);
    TenseurBH & operator = ( const TenseurBH &);
    TenseurBH & operator = ( const Tenseur1BH & B) 
      { return this->operator=(*((const TenseurBH*) & B)) ;};
    TenseurBH & operator * ( const double &) const ;
    void operator *= ( const double &);
    TenseurBH & operator / ( const double &) const ;
    void operator /= ( const double &);
 
    // produit contracte avec un vecteur
    CoordonneeB operator * ( const CoordonneeB & ) const ;
    // produit contracte contracté une fois A(i,j)*B(j,k)=A.B
    // -> donc c'est l'indice du milieu qui est contracté
    TenseurBB & operator * ( const TenseurBB &) const ; // produit une fois contracte
    TenseurBH & operator * ( const TenseurBH &) const ; // produit une fois contracte
    // produit contracte contracté deux fois A(i,j)*B(j,i)=A..B
    // -> on contracte d'abord l'indice du milieu puis l'indice externe
    double  operator && ( const TenseurBH &) const ; // produit deux fois contracte
    
    double  Trace() const ;   // trace du tenseur ou premier invariant
    double  II() const ;      // second invariant = trace (A*A)
    double  III() const ;     // troisieme invariant = trace ((A*A)*A)
    double  Det() const ;     // determinant de la matrice des coordonnees
    // valeurs propre  dans le vecteur  de retour, classée par ordres "décroissants"
    // cas indique le cas de valeur propre: 
    // quelque soit la dimension: cas = -1, si l'extraction des valeurs propres n'a pas pu se faire
    //                            dans ce cas les valeurs propres de retour sont nulles par défaut
    // dim = 1, cas=1 pour une valeur propre ; 
    virtual Coordonnee ValPropre(int& cas) const ;
    // idem met en retour la matrice mat contiend par colonne les vecteurs propre
    // elle doit avoir la dimension du tenseur
    // les vecteurs propre sont exprime dans le repere naturel  
    virtual Coordonnee ValPropre(int& cas, Mat_pleine& mat) const ;  

    // ici il s'agit uniquement de calculer les vecteurs propres, les valeurs propres
    // étant déjà connues
    // en retour VP les vecteurs propre : doivent avoir la dimension du tenseur
    // les vecteurs propre sont exprime dans le repere naturel (pour les tenseurs dim 3
    // pour dim=2:le premier vecteur propre est exprime dans le repere naturel 
    //            le second vecteur propre est exprimé dans le repère dual
    // pour dim=1 le vecteur est dans la base naturelle (mais ça importe pas)
    //   en sortie cas = -1 s'il y a eu un problème, dans ce cas, V_P est quelconque
    //             sinon si tout est ok, cas est identique en sortie avec l'entrée
    virtual void VecteursPropres(const Coordonnee& Val_P,int& cas, Tableau <Coordonnee>& V_P) const;
 
    // test
    int operator == ( const TenseurBH &) const ;
    int operator != ( const TenseurBH &) const ;

    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurBH & B,bool plusZero);
 
    // calcul du tenseur inverse par rapport au produit contracte
    TenseurBH & Inverse() const ;
    
    // tenseur transpose
    TenseurHB & Transpose() const ;  
    // symétrie formelle A^i_j = A_j^i : ==> création du tenseur en HB identique
    TenseurHB & Identique() const;
    // permute Bas Haut, mais reste dans le même tenseur
    // ici ne fait rien
    void PermuteHautBas() {};
    
    // calcul du maximum en valeur absolu des composantes du tenseur
     double MaxiComposante() const {return Dabs(*t);};
    
    // retourne la composante i,j en lecture/écriture
#ifdef MISE_AU_POINT
	 inline double& Coor(int i,int j)
#else 	
     inline double& Coor(int ,int )
#endif     	 
	 	{ 
         #ifdef MISE_AU_POINT	 	 
	    	if ( (i!=1) || (j!=1) )
			{ cout << "\nErreur : composante " << i <<"," << j <<" inexistante !\n";
			  cout << "Tenseur1BH::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
         #endif  			
		 return *t;
		}; 
		
    // retourne la composante i,j en lecture seule
#ifdef MISE_AU_POINT
	 inline double operator () (int i,int j) const 
#else 	
     inline double operator () (int ,int ) const
#endif     	 
	 	{ 
         #ifdef MISE_AU_POINT	 	 
	    	if ( (i!=1) || (j!=1) )
			{ cout << "\nErreur : composante " << i <<"," << j <<" inexistante !\n";
			  cout << "Tenseur1BH::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
         #endif  			
		 return *t;
		}; 
 
     //fonctions static définissant le produit tensoriel de deux vecteurs
     static TenseurBH & Prod_tensoriel(const CoordonneeB & aB, const CoordonneeH & bH);
	 
	 // lecture et écriture de données
     istream & Lecture(istream & entree);
     ostream & Ecriture(ostream & sort) const ;
	 
	 protected :
	    // allocator dans la liste de data
	  listdouble1Iter ipointe;  
		

    
};
/// @}  // end of group

/// @addtogroup Les_classes_tenseurs_dim1_ordre2
///  @{

//------------------------------------------------------------------
///  Definition des tenseur  derivees de dimension1.
///          cas des composantes mixtes HB
//------------------------------------------------------------------
/// \author    Gérard Rio
/// \version   1.0
/// \date       23/01/97
class Tenseur1HB : public TenseurHB
{
 // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1HB &);
 // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1HB &);
  public :
      // Constructeur
    Tenseur1HB() ; // par défaut
    // initialisation de la compoante (1,1)
    Tenseur1HB(double val) ;   
    // DESTRUCTEUR :
    ~Tenseur1HB();
    // constructeur a partir d'une instance non differenciee  
    Tenseur1HB (  const TenseurHB &); 
    // constructeur de copie  
    Tenseur1HB (  const Tenseur1HB & B);
        
    // METHODES PUBLIQUES :
    // initialise toutes les composantes à val
    void Inita(double val);   
    // operations 
    TenseurHB & operator + ( const TenseurHB &) const ;
    void operator += ( const TenseurHB &);
    TenseurHB & operator - () const ;
    TenseurHB & operator - ( const TenseurHB &) const ;
    void operator -= ( const TenseurHB &);
    TenseurHB & operator = ( const TenseurHB &) ;
    TenseurHB & operator = ( const Tenseur1HB & B) 
      { return this->operator=(*((const TenseurHB*) & B));};
    TenseurHB & operator * ( const double &) const ;
    void operator *= ( const double &);
    TenseurHB & operator / ( const double &) const ;
    void operator /= ( const double &);
    
    // produit contracte avec un vecteur
    CoordonneeH operator * ( const CoordonneeH & ) const ;
    
    // produit contracte contracté une fois A(i,j)*B(j,k)=A.B
    // -> donc c'est l'indice du milieu qui est contracté
    TenseurHH & operator * ( const TenseurHH &) const ;
    TenseurHB & operator * ( const TenseurHB &) const ; // produit une fois contracte
    // produit contracte contracté deux fois A(i,j)*B(j,i)=A..B
    // -> on contracte d'abord l'indice du milieu puis l'indice externe
    double  operator && ( const TenseurHB &) const ; // produit deux fois contracte
        
    double  Trace() const ;   // trace du tenseur ou premier invariant
    double  II() const ;      // second invariant = trace (A*A)
    double  III() const ;     // troisieme invariant = trace ((A*A)*A)
    double  Det() const ;     // determinant de la matrice des coordonnees
    // valeurs propre  dans le vecteur  de retour, classée par ordres "décroissants"
    // cas indique le cas de valeur propre: 
    // quelque soit la dimension: cas = -1, si l'extraction des valeurs propres n'a pas pu se faire
    //                            dans ce cas les valeurs propres de retour sont nulles par défaut
    // dim = 1, cas=1 pour une valeur propre ; 
    virtual Coordonnee ValPropre(int& cas) const ;
    // idem met en retour la matrice mat contiend par colonne les vecteurs propre
    // elle doit avoir la dimension du tenseur
    // les vecteurs propre sont exprime dans le repere naturel  
    virtual Coordonnee ValPropre(int& cas, Mat_pleine& mat) const ;  

    // ici il s'agit uniquement de calculer les vecteurs propres, les valeurs propres
    // étant déjà connues
    // en retour VP les vecteurs propre : doivent avoir la dimension du tenseur
    // les vecteurs propre sont exprime dans le repere naturel (pour les tenseurs dim 3
    // pour dim=2:le premier vecteur propre est exprime dans le repere naturel 
    //            le second vecteur propre est exprimé dans le repère dual
    // pour dim=1 le vecteur est dans la base naturelle (mais ça importe pas)
    //   en sortie cas = -1 s'il y a eu un problème, dans ce cas, V_P est quelconque
    //             sinon si tout est ok, cas est identique en sortie avec l'entrée
    virtual void VecteursPropres(const Coordonnee& Val_P,int& cas, Tableau <Coordonnee>& V_P) const;
 
    // test
    int operator == ( const TenseurHB &) const ;
    int operator != ( const TenseurHB &) const ;
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurHB & B,bool plusZero);

    // calcul du tenseur inverse par rapport au produit contracte
    TenseurHB & Inverse()  const ;
    
    // tenseur transpose
    TenseurBH & Transpose() const ; 
    // symétrie formelle A^i_j = A_j^i : ==> création du tenseur en BH identique
    TenseurBH & Identique() const;
    // permute Bas Haut, mais reste dans le même tenseur
    // ici ne fait rien
    void PermuteHautBas() {};
     
    // calcul du maximum en valeur absolu des composantes du tenseur
     double MaxiComposante() const {return Dabs(*t);};
                      
    // retourne la composante i,j en lecture/écriture
#ifdef MISE_AU_POINT
	 inline double& Coor(int i,int j)
#else 	
     inline double& Coor(int ,int )
#endif     	 
	 	{ 
          #ifdef MISE_AU_POINT	 	 
	    	if ( (i!=1) || (j!=1) )
		    	{ cout << "\nErreur : composante " << i <<"," << j <<" inexistante !\n";
		    	  cout << "Tenseur1HB::OPERATOR() (int,int ) \n";
			      Sortie(1);
			    };
          #endif  			
		  return *t;
		}; 
    // retourne la composante i,j en lecture seule
#ifdef MISE_AU_POINT
	 inline double operator () (int i,int j) const 
#else 	
     inline double operator () (int ,int ) const
#endif     	 
	 	{ 
          #ifdef MISE_AU_POINT	 	 
	    	if ( (i!=1) || (j!=1) )
		    	{ cout << "\nErreur : composante " << i <<"," << j <<" inexistante !\n";
		    	  cout << "Tenseur1HB::OPERATOR() (int,int ) \n";
			      Sortie(1);
			    };
          #endif  			
		  return *t;
		}; 
    
     //fonctions static définissant le produit tensoriel de deux vecteurs
     static TenseurHB & Prod_tensoriel(const CoordonneeH & aH, const CoordonneeB & bB);
	 
	 // lecture et écriture de données
     istream & Lecture(istream & entree);
     ostream & Ecriture(ostream & sort) const ;
	 
	 protected :
	    // allocator dans la liste de data
	  listdouble1Iter ipointe;  
		
};
/// @}  // end of group
 
#ifndef MISE_AU_POINT
  #include "Tenseur1-1.cc"
  #include "Tenseur1-2.cc"
  #define  Tenseur1_H_deja_inclus
#endif
     
 
#endif  
